@@ -29,7 +29,7 @@
 
 #include "oci_wrapper.h"
 
-CPL_CVSID("$Id: oci_wrapper.cpp 499f2eb9c7e58d30cb5a11ce5d88589a44a4df7b 2018-10-19 11:35:28 -0700 Fengting Chen $")
+CPL_CVSID("$Id: oci_wrapper.cpp bf3a27bbd1f495a7e83d7373545798549a9f813e 2021-05-04 14:06:34 -0400 fechen123 $")
 
 static const OW_CellDepth ahOW_CellDepth[] = {
     {"8BIT_U",          GDT_Byte},
@@ -56,20 +56,6 @@ OWConnection::OWConnection( OCIExtProcContext* poWithContext )
     pszPassword     = CPLStrdup( "" );
     pszServer       = CPLStrdup( "" );
 
-    hEnv            = nullptr;
-    hError          = nullptr;
-    hSvcCtx         = nullptr;
-    hSession        = nullptr;
-    hDescribe       = nullptr;
-    hNumArrayTDO    = nullptr;
-    hGeometryTDO    = nullptr;
-    hGeoRasterTDO   = nullptr;
-    hElemArrayTDO   = nullptr;
-    hOrdnArrayTDO   = nullptr;
-    bSuceeeded      = false;
-    nCharSize       = 1;
-    bExtProc        = false;
-
     if( ! poWithContext )
     {
        return;
@@ -105,7 +91,7 @@ OWConnection::OWConnection( OCIExtProcContext* poWithContext )
     poStmt->Define(szUser);
     poStmt->Define(szSchema);
 
-    poStmt->Execute();
+    CPL_IGNORE_RET_VAL(poStmt->Execute());
 
     pszExtProcSchema = CPLStrdup( szSchema );
     pszExtProcUser   = CPLStrdup( szUser );
@@ -128,20 +114,6 @@ OWConnection::OWConnection( const char* pszUserIn,
     pszPassword     = CPLStrdup( pszPasswordIn );
     pszServer       = CPLStrdup( pszServerIn );
 
-    hEnv            = nullptr;
-    hError          = nullptr;
-    hSvcCtx         = nullptr;
-    hSession        = nullptr;
-    hDescribe       = nullptr;
-    hNumArrayTDO    = nullptr;
-    hGeometryTDO    = nullptr;
-    hGeoRasterTDO   = nullptr;
-    hElemArrayTDO   = nullptr;
-    hOrdnArrayTDO   = nullptr;
-    bSuceeeded      = false;
-    nCharSize       = 1;
-    bExtProc        = false;
-
     // ------------------------------------------------------
     //  Operational Systems's authentication option
     // ------------------------------------------------------
@@ -150,8 +122,7 @@ OWConnection::OWConnection( const char* pszUserIn,
 
     ub4 eCred = OCI_CRED_RDBMS;
 
-    if( EQUAL(pszServer, "") &&
-        EQUAL(pszPassword, "") &&
+    if( EQUAL(pszPassword, "") &&
         EQUAL(pszUser, "") )
     {
         eCred = OCI_CRED_EXT;
@@ -266,6 +237,22 @@ OWConnection::OWConnection( const char* pszUserIn,
 
     bSuceeeded = true;
 
+    // ------------------------------------------------------
+    //  If no user specified, get it from the session 
+    // ------------------------------------------------------
+    if (EQUAL(pszUser, "") )
+    {
+      OWStatement* poStmt = CreateStatement(
+            "select sys_context('userenv','session_user')\n"
+            "from dual\n" );
+
+      pszUser = static_cast<char*>(CPLRealloc(pszUser, OWNAME)); 
+      poStmt->Define(pszUser);
+      CPL_IGNORE_RET_VAL(poStmt->Execute());
+      delete poStmt;
+      CPLDebug("OCI: ", "Implicit User: %s\n", pszUser);
+    }
+
     // ------------------------------------------------------
     //  Initialize/Describe types
     // ------------------------------------------------------
@@ -325,6 +312,8 @@ OWConnection::~OWConnection()
     DestroyType( hGeoRasterTDO );
     DestroyType( hElemArrayTDO );
     DestroyType( hOrdnArrayTDO );
+    if( hPCTDO )
+        DestroyType( hPCTDO );
 
     OCIHandleFree( (dvoid*) hDescribe, (ub4) OCI_HTYPE_DESCRIBE);
 
@@ -370,6 +359,9 @@ OWConnection::~OWConnection()
     {
         OCIHandleFree((dvoid *) hSession, (ub4) OCI_HTYPE_SESSION);
     }
+
+    CPLFree( pszExtProcUser );
+    CPLFree( pszExtProcSchema );
 }
 
 OCIType* OWConnection::DescribeType( const char *pszTypeName )
@@ -540,7 +532,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
                                  int* pnPrecision,
                                  signed short* pnScale )
 {
-    OCIParam* hParmDesc = nullptr;
+    OCIParam* hParamDesc = nullptr;
 
     sword nStatus = 0;
 
@@ -548,7 +540,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
         phTable,
         (ub4) OCI_DTYPE_PARAM,
         hError,
-        (dvoid**) &hParmDesc,   //Warning
+        (dvoid**) &hParamDesc,   //Warning
         (ub4) nIndex + 1 );
 
     if( nStatus != OCI_SUCCESS )
@@ -560,7 +552,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
     ub4 nNameLength = 0;
 
     CheckError( OCIAttrGet(
-        hParmDesc,
+        hParamDesc,
         (ub4) OCI_DTYPE_PARAM,
         (dvoid*) &pszFieldName,
         (ub4*) &nNameLength,
@@ -570,7 +562,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
     ub2 nOCIType = 0;
 
     CheckError( OCIAttrGet(
-        hParmDesc,
+        hParamDesc,
         (ub4) OCI_DTYPE_PARAM,
         (dvoid*) &nOCIType,
         (ub4*) nullptr,
@@ -580,7 +572,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
     ub2 nOCILen = 0;
 
     CheckError( OCIAttrGet(
-        hParmDesc,
+        hParamDesc,
         (ub4) OCI_DTYPE_PARAM,
         (dvoid*) &nOCILen,
         (ub4*) nullptr,
@@ -593,7 +585,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
     if( nOCIType == SQLT_NUM )
     {
         CheckError( OCIAttrGet(
-            hParmDesc,
+            hParamDesc,
             (ub4) OCI_DTYPE_PARAM,
             (dvoid*) &nOCIPrecision,
             (ub4*) nullptr,
@@ -601,7 +593,7 @@ bool OWConnection::GetNextField( OCIParam* phTable,
             hError ), hError );
 
         CheckError( OCIAttrGet(
-            hParmDesc,
+            hParamDesc,
             (ub4) OCI_DTYPE_PARAM,
             (dvoid*) &nOCIScale,
             (ub4*) nullptr,
@@ -1565,7 +1557,7 @@ char* OWStatement::ReadCLob( OCILobLocator* phLocator )
 
     nSize *= this->poConnection->nCharSize;
 
-    pszBuffer = (char*) VSIMalloc( sizeof(char*) * nSize );
+    pszBuffer = (char*) VSICalloc( 1, nSize + 1 );
 
     if( pszBuffer == nullptr)
     {
@@ -1870,8 +1862,7 @@ void OWUpperIfNoQuotes( char* pszText )
 /*****************************************************************************/
 /*                     Parse Value after a Hint on a string                  */
 /*****************************************************************************/
-static
-const char *OWParseValue( const char* pszText,
+static CPLString OWParseValue( const char* pszText,
                           const char* pszSeparators,
                           const char* pszHint,
                           int nOffset )
@@ -1885,20 +1876,20 @@ const char *OWParseValue( const char* pszText,
         CSLT_PRESERVEQUOTES );
 
     nCount = CSLCount( papszTokens );
-    const char* pszResult = "";
+    CPLString osResult;
 
     for( i = 0; ( i + nOffset ) < nCount; i++ )
     {
         if( EQUAL( papszTokens[i], pszHint ) )
         {
-            pszResult = CPLStrdup( papszTokens[i + nOffset] );
+            osResult = papszTokens[i + nOffset];
             break;
         }
     }
 
     CSLDestroy( papszTokens );
 
-    return pszResult;
+    return osResult;
 }
 
 /*****************************************************************************/
@@ -1911,12 +1902,12 @@ const char *OWParseValue( const char* pszText,
  *
  */
 
-const char* OWParseSDO_GEOR_INIT( const char* pszInsert, int nField )
+CPLString OWParseSDO_GEOR_INIT( const char* pszInsert, int nField )
 {
     char  szUpcase[OWTEXT];
     char* pszIn = nullptr;
 
-    strcpy( szUpcase, pszInsert );
+    snprintf( szUpcase, sizeof(szUpcase), "%s", pszInsert );
 
     for( pszIn = szUpcase; *pszIn != '\0'; pszIn++ )
     {
@@ -1948,9 +1939,9 @@ const char* OWParseSDO_GEOR_INIT( const char* pszInsert, int nField )
     strncpy( szBuffer, pszStart, nLength );
     szBuffer[nLength] = '\0';
 
-    const char* pszValue = OWParseValue( szBuffer, " (,)", "INIT", nField );
+    auto osValue = OWParseValue( szBuffer, " (,)", "INIT", nField );
 
-    return EQUAL( pszValue, "" ) ? "NULL" : pszValue;
+    return EQUAL( osValue, "" ) ? "NULL" : osValue;
 }
 
 /*****************************************************************************/
@@ -1966,9 +1957,7 @@ const char* OWParseSDO_GEOR_INIT( const char* pszInsert, int nField )
 
 int OWParseServerVersion( const char* pszText )
 {
-    const char* pszValue = OWParseValue( pszText, " .", "Release", 1 );
-
-    return pszValue == nullptr ? 0 : atoi( pszValue );
+    return atoi(OWParseValue( pszText, " .", "Release", 1 ));
 }
 
 /*****************************************************************************/
@@ -1984,9 +1973,7 @@ int OWParseServerVersion( const char* pszText )
 
 int OWParseEPSG( const char* pszText )
 {
-    const char* pszValue = OWParseValue( pszText, " ()", "EPSG", 2 );
-
-    return pszValue == nullptr ? 0 : atoi( pszValue );
+    return atoi(OWParseValue( pszText, " ()", "EPSG", 2 ));
 }
 
 /*****************************************************************************/
