@@ -44,7 +44,7 @@
 #ifdef GDAL_SUPPORT
 #include "cpl_vsi.h"
 
-CPL_CVSID("$Id: msg_reader_core.cpp 36776 2016-12-10 11:17:47Z rouault $");
+CPL_CVSID("$Id: msg_reader_core.cpp a6bcda3760ce7fc7ecf5e16e78394ba12475d698 2021-07-05 22:23:51 +0200 Even Rouault $")
 
 #else
 #define VSIFSeek(fp, pos, ref)    CPL_IGNORE_RET_VAL(fseek(fp, pos, ref))
@@ -119,13 +119,14 @@ Msg_reader_core::Msg_reader_core( const char* fname ) :
     _open_success(false)
 {
     memset(&_main_header, 0, sizeof(_main_header));
+    memset(&_sec_header, 0, sizeof(_sec_header));
     SecondaryProdHeaderInit(&_sec_header);
     for (size_t i=0; i < MSG_NUM_CHANNELS; ++i) {
       _calibration[i].cal_slope = 0.0;
       _calibration[i].cal_offset = 0.0;
     }
 
-    FILE* fin = fopen(fname, "rb");
+    VSILFILE* fin = VSIFOpenL(fname, "rb");
     if( !fin )
     {
         CPLError(CE_Failure, CPLE_OpenFailed,
@@ -133,10 +134,10 @@ Msg_reader_core::Msg_reader_core( const char* fname ) :
         return;
     }
     read_metadata_block(fin);
-    fclose(fin);
+    VSIFCloseL(fin);
 }
 
-Msg_reader_core::Msg_reader_core( FILE* fp ) :
+Msg_reader_core::Msg_reader_core( VSILFILE* fp ) :
     _lines(0),
     _columns(0),
     _line_start(0),
@@ -160,6 +161,7 @@ Msg_reader_core::Msg_reader_core( FILE* fp ) :
     _open_success(false)
 {
     memset(&_main_header, 0, sizeof(_main_header));
+    memset(&_sec_header, 0, sizeof(_sec_header));
 
     SecondaryProdHeaderInit(&_sec_header);
     for (size_t i=0; i < MSG_NUM_CHANNELS; ++i) {
@@ -170,13 +172,13 @@ Msg_reader_core::Msg_reader_core( FILE* fp ) :
     read_metadata_block(fp);
 }
 
-void Msg_reader_core::read_metadata_block(FILE* fin) {
+void Msg_reader_core::read_metadata_block(VSILFILE* fin) {
     _open_success = true;
 
     unsigned int i;
 
-    CPL_IGNORE_RET_VAL(VSIFRead(&_main_header, sizeof(_main_header), 1, fin));
-    CPL_IGNORE_RET_VAL(VSIFRead(&_sec_header, sizeof(_sec_header), 1, fin));
+    CPL_IGNORE_RET_VAL(VSIFReadL(&_main_header, sizeof(_main_header), 1, fin));
+    CPL_IGNORE_RET_VAL(VSIFReadL(&_sec_header, sizeof(_sec_header), 1, fin));
 
 #ifdef DEBUG
     // print out all the fields in the header
@@ -222,23 +224,27 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
     sscanf(_sec_header.northLineSelectedRectangle.value, "%u", &_lines);
     sscanf(_sec_header.southLineSelectedRectangle.value, "%u", &lines);
     _line_start = lines;
-    _lines -= lines - 1;
+    if( lines > 0 && _lines >= lines - 1 )
+        _lines -= lines - 1;
 
     unsigned int cols;
     sscanf(_sec_header.westColumnSelectedRectangle.value, "%u", &_columns);
     sscanf(_sec_header.eastColumnSelectedRectangle.value, "%u", &cols);
     _col_start = cols;
-    _columns -= cols - 1;
+    if( cols > 0 && _columns >= cols - 1 )
+        _columns -= cols - 1;
 
 #ifdef DEBUG
     printf("lines = %u, cols = %u\n", _lines, _columns);/*ok*/
+    int records_per_line = 0;
 #endif // DEBUG
 
-    int records_per_line = 0;
     for (i=0; i < MSG_NUM_CHANNELS; i++) {
         if (_sec_header.selectedBandIds.value[i] == 'X') {
             _bands[i] = 1;
+#ifdef DEBUG
             records_per_line += (i == (MSG_NUM_CHANNELS-1)) ? 3 : 1;
+#endif // DEBUG
         } else {
             _bands[i] = 0;
         }
@@ -258,8 +264,8 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
     // read radiometric block
     RADIOMETRIC_PROCESSING_RECORD rad;
     off_t offset = RADIOMETRICPROCESSING_RECORD_OFFSET + _f_header_offset + sizeof(GP_PK_HEADER) + sizeof(GP_PK_SH1) + 1;
-    CPL_IGNORE_RET_VAL(VSIFSeek(fin, offset, SEEK_SET));
-    CPL_IGNORE_RET_VAL(VSIFRead(&rad, sizeof(RADIOMETRIC_PROCESSING_RECORD), 1, fin));
+    CPL_IGNORE_RET_VAL(VSIFSeekL(fin, offset, SEEK_SET));
+    CPL_IGNORE_RET_VAL(VSIFReadL(&rad, sizeof(RADIOMETRIC_PROCESSING_RECORD), 1, fin));
     to_native(rad);
     memcpy((void*)_calibration, (void*)&rad.level1_5ImageCalibration,sizeof(_calibration));
 
@@ -278,8 +284,8 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
     // read image description block
     IMAGE_DESCRIPTION_RECORD idr;
     offset = RADIOMETRICPROCESSING_RECORD_OFFSET  - IMAGEDESCRIPTION_RECORD_LENGTH + _f_header_offset + sizeof(GP_PK_HEADER) + sizeof(GP_PK_SH1) + 1;
-    CPL_IGNORE_RET_VAL(VSIFSeek(fin, offset, SEEK_SET));
-    CPL_IGNORE_RET_VAL(VSIFRead(&idr, sizeof(IMAGE_DESCRIPTION_RECORD), 1, fin));
+    CPL_IGNORE_RET_VAL(VSIFSeekL(fin, offset, SEEK_SET));
+    CPL_IGNORE_RET_VAL(VSIFReadL(&idr, sizeof(IMAGE_DESCRIPTION_RECORD), 1, fin));
     to_native(idr);
     _line_dir_step = idr.referencegrid_visir.lineDirGridStep;
     _col_dir_step = idr.referencegrid_visir.columnDirGridStep;
@@ -290,7 +296,7 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
     GP_PK_SH1    sub_header;
     SUB_VISIRLINE visir_line;
 
-    CPL_IGNORE_RET_VAL(VSIFSeek(fin, _f_data_offset, SEEK_SET));
+    CPL_IGNORE_RET_VAL(VSIFSeekL(fin, _f_data_offset, SEEK_SET));
 
     _hrv_packet_size = 0;
     _interline_spacing = 0;
@@ -304,14 +310,18 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
     }
 
     do {
-        CPL_IGNORE_RET_VAL(VSIFRead(&gp_header, sizeof(GP_PK_HEADER), 1, fin));
-        CPL_IGNORE_RET_VAL(VSIFRead(&sub_header, sizeof(GP_PK_SH1), 1, fin));
-        CPL_IGNORE_RET_VAL(VSIFRead(&visir_line, sizeof(SUB_VISIRLINE), 1, fin));
+        if( VSIFReadL(&gp_header, sizeof(GP_PK_HEADER), 1, fin) != 1 ||
+            VSIFReadL(&sub_header, sizeof(GP_PK_SH1), 1, fin) != 1 ||
+            VSIFReadL(&visir_line, sizeof(SUB_VISIRLINE), 1, fin) != 1 )
+        {
+            _open_success = false;
+            break;
+        }
         to_native(visir_line);
         to_native(gp_header);
 
         // skip over the actual line data
-        CPL_IGNORE_RET_VAL(VSIFSeek(fin,
+        CPL_IGNORE_RET_VAL(VSIFSeekL(fin,
             gp_header.packetLength - (sizeof(GP_PK_SH1) + sizeof(SUB_VISIRLINE) - 1),
             SEEK_CUR
         ));
@@ -333,7 +343,7 @@ void Msg_reader_core::read_metadata_block(FILE* fin) {
                 _hrv_bytes_per_line = gp_header.packetLength - (unsigned int)(sizeof(GP_PK_SH1) + sizeof(SUB_VISIRLINE) - 1);
                 _hrv_packet_size = gp_header.packetLength + (unsigned int)sizeof(GP_PK_HEADER) + 1;
                 _interline_spacing +=  3*_hrv_packet_size;
-                CPL_IGNORE_RET_VAL(VSIFSeek(fin, 2*gp_header.packetLength, SEEK_CUR ));
+                CPL_IGNORE_RET_VAL(VSIFSeekL(fin, 2*gp_header.packetLength, SEEK_CUR ));
             }
         }
     } while (band_count > 0);
@@ -352,7 +362,7 @@ int Msg_reader_core::_chan_to_idx(Msg_channel_names channel) {
     return 0;
 }
 
-void Msg_reader_core::get_pixel_geo_coordinates(unsigned int line, unsigned int column, double& longitude, double& latitude) {
+void Msg_reader_core::get_pixel_geo_coordinates(unsigned int line, unsigned int column, double& longitude, double& latitude) const {
     Conversions::convert_pixel_to_geo((unsigned int)(line + _line_start), (unsigned int)(column + _col_start), longitude, latitude);
 }
 
