@@ -33,7 +33,7 @@
 #include "cpl_string.h"
 #include <vector>
 
-CPL_CVSID("$Id: ogrgeomediadatasource.cpp 657d65d2f09c031221875536844191be01b4ea66 2020-08-31 18:09:29 +1000 Nyall Dawson $")
+CPL_CVSID("$Id: ogrgeomediadatasource.cpp 316de08882c4b40ed999aa6e37198a5168b8e93c 2021-08-23 11:12:04 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                       OGRGeomediaDataSource()                        */
@@ -140,6 +140,27 @@ int OGRGeomediaDataSource::Open( const char * pszNewName, int bUpdate,
         }
     }
 
+    // check for GAliasTable table
+    {
+        bool bFoundGAliasTable = false;
+        CPLODBCStatement oTableList( &oSession );
+        if( oTableList.GetTables() )
+        {
+            while( oTableList.Fetch() )
+            {
+                const CPLString osTableName = CPLString( oTableList.GetColData(2, "") );
+                const CPLString osLCTableName(CPLString(osTableName).tolower());
+                if( osLCTableName == "galiastable" )
+                {
+                    bFoundGAliasTable = true;
+                    break;
+                }
+            }
+        }
+        if (!bFoundGAliasTable )
+            return FALSE;
+    }
+
     pszName = CPLStrdup( pszNewName );
 
     bDSUpdate = bUpdate;
