@@ -40,7 +40,7 @@
 #include "cpl_time.h"
 #include "swq_parser.hpp"
 
-CPL_CVSID("$Id: swq.cpp 6659c4d5c910df313ac451cd5ca687862eef040b 2020-10-01 15:37:53 +0200 Even Rouault $")
+CPL_CVSID("$Id: swq.cpp 90e8da8c0a56e2d99683b89ad1c242680d55ce76 2021-06-01 20:28:01 +0200 Even Rouault $")
 
 #define YYSTYPE swq_expr_node *
 
@@ -779,7 +779,10 @@ static void swq_fixup_expression(swq_expr_node* node)
             {
                 std::vector<swq_expr_node*> exprs;
                 for( int i = 0; i < node->nSubExprCount; i++ )
+                {
+                    swq_fixup_expression(node->papoSubExpr[i]);
                     exprs.push_back(node->papoSubExpr[i]);
+                }
                 node->nSubExprCount = 0;
                 CPLFree( node->papoSubExpr );
                 node->papoSubExpr = nullptr;
@@ -841,6 +844,65 @@ void swq_fixup(swq_parse_context* psParseContext)
     }
 }
 
+/************************************************************************/
+/*                       swq_create_and_or_or()                         */
+/************************************************************************/
+
+swq_expr_node* swq_create_and_or_or(swq_op op, swq_expr_node* left, swq_expr_node* right)
+{
+    auto poNode = new swq_expr_node( op );
+    poNode->field_type = SWQ_BOOLEAN;
+
+    if( left->eNodeType == SNT_OPERATION &&
+        left->nOperation == op  )
+    {
+        // Temporary non-binary formulation
+        if( right->eNodeType == SNT_OPERATION &&
+            right->nOperation == op  )
+        {
+            poNode->nSubExprCount = left->nSubExprCount + right->nSubExprCount;
+            poNode->papoSubExpr = static_cast<swq_expr_node **>(
+                CPLRealloc( left->papoSubExpr, sizeof(swq_expr_node*) * poNode->nSubExprCount ));
+            memcpy(poNode->papoSubExpr + left->nSubExprCount,
+                   right->papoSubExpr,
+                   right->nSubExprCount * sizeof(swq_expr_node*));
+
+            right->nSubExprCount = 0;
+            right->papoSubExpr = nullptr;
+            delete right;
+        }
+        else
+        {
+            poNode->nSubExprCount = left->nSubExprCount;
+            poNode->papoSubExpr = left->papoSubExpr;
+            poNode->PushSubExpression( right );
+        }
+
+        left->nSubExprCount = 0;
+        left->papoSubExpr = nullptr;
+        delete left;
+    }
+    else if( right->eNodeType == SNT_OPERATION &&
+             right->nOperation == op  )
+    {
+        // Temporary non-binary formulation
+        poNode->nSubExprCount = right->nSubExprCount;
+        poNode->papoSubExpr = right->papoSubExpr;
+        poNode->PushSubExpression( left );
+
+        right->nSubExprCount = 0;
+        right->papoSubExpr = nullptr;
+        delete right;
+    }
+    else
+    {
+        poNode->PushSubExpression( left );
+        poNode->PushSubExpression( right );
+    }
+
+    return poNode;
+}
+
 /************************************************************************/
 /*                         swq_expr_compile2()                          */
 /************************************************************************/
